from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.administrative_information import AdministrativeInformation
    from ..models.endpoint import Endpoint
    from ..models.extension import Extension
    from ..models.lang_string_name_type import LangStringNameType
    from ..models.lang_string_text_type import LangStringTextType
    from ..models.reference import Reference


T = TypeVar("T", bound="SubmodelDescriptor")


@_attrs_define
class SubmodelDescriptor:
    """
    Attributes:
        id (str):
        endpoints (List['Endpoint']):
        description (Union[Unset, List['LangStringTextType']]):
        display_name (Union[Unset, List['LangStringNameType']]):
        extensions (Union[Unset, List['Extension']]):
        administration (Union[Unset, AdministrativeInformation]):
        id_short (Union[Unset, str]):
        semantic_id (Union[Unset, Reference]):
        supplemental_semantic_id (Union[Unset, List['Reference']]):
    """

    id: str
    endpoints: List["Endpoint"]
    description: Union[Unset, List["LangStringTextType"]] = UNSET
    display_name: Union[Unset, List["LangStringNameType"]] = UNSET
    extensions: Union[Unset, List["Extension"]] = UNSET
    administration: Union[Unset, "AdministrativeInformation"] = UNSET
    id_short: Union[Unset, str] = UNSET
    semantic_id: Union[Unset, "Reference"] = UNSET
    supplemental_semantic_id: Union[Unset, List["Reference"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        endpoints = []
        for endpoints_item_data in self.endpoints:
            endpoints_item = endpoints_item_data.to_dict()
            endpoints.append(endpoints_item)

        description: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.description, Unset):
            description = []
            for description_item_data in self.description:
                description_item = description_item_data.to_dict()
                description.append(description_item)

        display_name: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.display_name, Unset):
            display_name = []
            for display_name_item_data in self.display_name:
                display_name_item = display_name_item_data.to_dict()
                display_name.append(display_name_item)

        extensions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.extensions, Unset):
            extensions = []
            for extensions_item_data in self.extensions:
                extensions_item = extensions_item_data.to_dict()
                extensions.append(extensions_item)

        administration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.administration, Unset):
            administration = self.administration.to_dict()

        id_short = self.id_short

        semantic_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.semantic_id, Unset):
            semantic_id = self.semantic_id.to_dict()

        supplemental_semantic_id: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.supplemental_semantic_id, Unset):
            supplemental_semantic_id = []
            for supplemental_semantic_id_item_data in self.supplemental_semantic_id:
                supplemental_semantic_id_item = supplemental_semantic_id_item_data.to_dict()
                supplemental_semantic_id.append(supplemental_semantic_id_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "endpoints": endpoints,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if extensions is not UNSET:
            field_dict["extensions"] = extensions
        if administration is not UNSET:
            field_dict["administration"] = administration
        if id_short is not UNSET:
            field_dict["idShort"] = id_short
        if semantic_id is not UNSET:
            field_dict["semanticId"] = semantic_id
        if supplemental_semantic_id is not UNSET:
            field_dict["supplementalSemanticId"] = supplemental_semantic_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.administrative_information import AdministrativeInformation
        from ..models.endpoint import Endpoint
        from ..models.extension import Extension
        from ..models.lang_string_name_type import LangStringNameType
        from ..models.lang_string_text_type import LangStringTextType
        from ..models.reference import Reference

        d = src_dict.copy()
        id = d.pop("id")

        endpoints = []
        _endpoints = d.pop("endpoints")
        for endpoints_item_data in _endpoints:
            endpoints_item = Endpoint.from_dict(endpoints_item_data)

            endpoints.append(endpoints_item)

        description = []
        _description = d.pop("description", UNSET)
        for description_item_data in _description or []:
            description_item = LangStringTextType.from_dict(description_item_data)

            description.append(description_item)

        display_name = []
        _display_name = d.pop("displayName", UNSET)
        for display_name_item_data in _display_name or []:
            display_name_item = LangStringNameType.from_dict(display_name_item_data)

            display_name.append(display_name_item)

        extensions = []
        _extensions = d.pop("extensions", UNSET)
        for extensions_item_data in _extensions or []:
            extensions_item = Extension.from_dict(extensions_item_data)

            extensions.append(extensions_item)

        _administration = d.pop("administration", UNSET)
        administration: Union[Unset, AdministrativeInformation]
        if isinstance(_administration, Unset):
            administration = UNSET
        else:
            administration = AdministrativeInformation.from_dict(_administration)

        id_short = d.pop("idShort", UNSET)

        _semantic_id = d.pop("semanticId", UNSET)
        semantic_id: Union[Unset, Reference]
        if isinstance(_semantic_id, Unset):
            semantic_id = UNSET
        else:
            semantic_id = Reference.from_dict(_semantic_id)

        supplemental_semantic_id = []
        _supplemental_semantic_id = d.pop("supplementalSemanticId", UNSET)
        for supplemental_semantic_id_item_data in _supplemental_semantic_id or []:
            supplemental_semantic_id_item = Reference.from_dict(supplemental_semantic_id_item_data)

            supplemental_semantic_id.append(supplemental_semantic_id_item)

        submodel_descriptor = cls(
            id=id,
            endpoints=endpoints,
            description=description,
            display_name=display_name,
            extensions=extensions,
            administration=administration,
            id_short=id_short,
            semantic_id=semantic_id,
            supplemental_semantic_id=supplemental_semantic_id,
        )

        submodel_descriptor.additional_properties = d
        return submodel_descriptor

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.page import Page
    from ..models.shell_descriptor_query import ShellDescriptorQuery
    from ..models.sorting import Sorting


T = TypeVar("T", bound="ShellDescriptorSearchRequest")


@_attrs_define
class ShellDescriptorSearchRequest:
    """
    Attributes:
        page (Union[Unset, Page]):
        sort_by (Union[Unset, Sorting]):
        query (Union[Unset, ShellDescriptorQuery]):
    """

    page: Union[Unset, "Page"] = UNSET
    sort_by: Union[Unset, "Sorting"] = UNSET
    query: Union[Unset, "ShellDescriptorQuery"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        page: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.page, Unset):
            page = self.page.to_dict()

        sort_by: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sort_by, Unset):
            sort_by = self.sort_by.to_dict()

        query: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.query, Unset):
            query = self.query.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if page is not UNSET:
            field_dict["page"] = page
        if sort_by is not UNSET:
            field_dict["sortBy"] = sort_by
        if query is not UNSET:
            field_dict["query"] = query

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.page import Page
        from ..models.shell_descriptor_query import ShellDescriptorQuery
        from ..models.sorting import Sorting

        d = src_dict.copy()
        _page = d.pop("page", UNSET)
        page: Union[Unset, Page]
        if isinstance(_page, Unset):
            page = UNSET
        else:
            page = Page.from_dict(_page)

        _sort_by = d.pop("sortBy", UNSET)
        sort_by: Union[Unset, Sorting]
        if isinstance(_sort_by, Unset):
            sort_by = UNSET
        else:
            sort_by = Sorting.from_dict(_sort_by)

        _query = d.pop("query", UNSET)
        query: Union[Unset, ShellDescriptorQuery]
        if isinstance(_query, Unset):
            query = UNSET
        else:
            query = ShellDescriptorQuery.from_dict(_query)

        shell_descriptor_search_request = cls(
            page=page,
            sort_by=sort_by,
            query=query,
        )

        shell_descriptor_search_request.additional_properties = d
        return shell_descriptor_search_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProtocolInformationSecurityAttributesType(str, Enum):
    NONE = "NONE"
    RFC_TLSA = "RFC_TLSA"
    W3C_DID = "W3C_DID"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="GetAllSubmodelDescriptorsThroughSuperpathAasIdentifier")


@_attrs_define
class GetAllSubmodelDescriptorsThroughSuperpathAasIdentifier:
    """
    Attributes:
        short (Union[Unset, int]):
        char (Union[Unset, str]):
        int_ (Union[Unset, int]):
        long (Union[Unset, int]):
        float_ (Union[Unset, float]):
        double (Union[Unset, float]):
        direct (Union[Unset, bool]):
        read_only (Union[Unset, bool]):
    """

    short: Union[Unset, int] = UNSET
    char: Union[Unset, str] = UNSET
    int_: Union[Unset, int] = UNSET
    long: Union[Unset, int] = UNSET
    float_: Union[Unset, float] = UNSET
    double: Union[Unset, float] = UNSET
    direct: Union[Unset, bool] = UNSET
    read_only: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        short = self.short

        char = self.char

        int_ = self.int_

        long = self.long

        float_ = self.float_

        double = self.double

        direct = self.direct

        read_only = self.read_only

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if short is not UNSET:
            field_dict["short"] = short
        if char is not UNSET:
            field_dict["char"] = char
        if int_ is not UNSET:
            field_dict["int"] = int_
        if long is not UNSET:
            field_dict["long"] = long
        if float_ is not UNSET:
            field_dict["float"] = float_
        if double is not UNSET:
            field_dict["double"] = double
        if direct is not UNSET:
            field_dict["direct"] = direct
        if read_only is not UNSET:
            field_dict["readOnly"] = read_only

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        short = d.pop("short", UNSET)

        char = d.pop("char", UNSET)

        int_ = d.pop("int", UNSET)

        long = d.pop("long", UNSET)

        float_ = d.pop("float", UNSET)

        double = d.pop("double", UNSET)

        direct = d.pop("direct", UNSET)

        read_only = d.pop("readOnly", UNSET)

        get_all_submodel_descriptors_through_superpath_aas_identifier = cls(
            short=short,
            char=char,
            int_=int_,
            long=long,
            float_=float_,
            double=double,
            direct=direct,
            read_only=read_only,
        )

        get_all_submodel_descriptors_through_superpath_aas_identifier.additional_properties = d
        return get_all_submodel_descriptors_through_superpath_aas_identifier

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SortingPathItem(str, Enum):
    ADMINISTRATION_REVISION = "administration.revision"
    ADMINISTRATION_VERSION = "administration.version"
    ID = "id"
    IDSHORT = "idShort"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.reference import Reference


T = TypeVar("T", bound="SpecificAssetId")


@_attrs_define
class SpecificAssetId:
    """
    Attributes:
        name (str):
        value (str):
        semantic_id (Union[Unset, Reference]):
        supplemental_semantic_ids (Union[Unset, List['Reference']]):
        external_subject_id (Union[Unset, Reference]):
    """

    name: str
    value: str
    semantic_id: Union[Unset, "Reference"] = UNSET
    supplemental_semantic_ids: Union[Unset, List["Reference"]] = UNSET
    external_subject_id: Union[Unset, "Reference"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        value = self.value

        semantic_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.semantic_id, Unset):
            semantic_id = self.semantic_id.to_dict()

        supplemental_semantic_ids: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.supplemental_semantic_ids, Unset):
            supplemental_semantic_ids = []
            for supplemental_semantic_ids_item_data in self.supplemental_semantic_ids:
                supplemental_semantic_ids_item = supplemental_semantic_ids_item_data.to_dict()
                supplemental_semantic_ids.append(supplemental_semantic_ids_item)

        external_subject_id: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.external_subject_id, Unset):
            external_subject_id = self.external_subject_id.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "value": value,
            }
        )
        if semantic_id is not UNSET:
            field_dict["semanticId"] = semantic_id
        if supplemental_semantic_ids is not UNSET:
            field_dict["supplementalSemanticIds"] = supplemental_semantic_ids
        if external_subject_id is not UNSET:
            field_dict["externalSubjectId"] = external_subject_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.reference import Reference

        d = src_dict.copy()
        name = d.pop("name")

        value = d.pop("value")

        _semantic_id = d.pop("semanticId", UNSET)
        semantic_id: Union[Unset, Reference]
        if isinstance(_semantic_id, Unset):
            semantic_id = UNSET
        else:
            semantic_id = Reference.from_dict(_semantic_id)

        supplemental_semantic_ids = []
        _supplemental_semantic_ids = d.pop("supplementalSemanticIds", UNSET)
        for supplemental_semantic_ids_item_data in _supplemental_semantic_ids or []:
            supplemental_semantic_ids_item = Reference.from_dict(supplemental_semantic_ids_item_data)

            supplemental_semantic_ids.append(supplemental_semantic_ids_item)

        _external_subject_id = d.pop("externalSubjectId", UNSET)
        external_subject_id: Union[Unset, Reference]
        if isinstance(_external_subject_id, Unset):
            external_subject_id = UNSET
        else:
            external_subject_id = Reference.from_dict(_external_subject_id)

        specific_asset_id = cls(
            name=name,
            value=value,
            semantic_id=semantic_id,
            supplemental_semantic_ids=supplemental_semantic_ids,
            external_subject_id=external_subject_id,
        )

        specific_asset_id.additional_properties = d
        return specific_asset_id

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class KeyType(str, Enum):
    ANNOTATEDRELATIONSHIPELEMENT = "AnnotatedRelationshipElement"
    ASSETADMINISTRATIONSHELL = "AssetAdministrationShell"
    BASICEVENTELEMENT = "BasicEventElement"
    BLOB = "Blob"
    CAPABILITY = "Capability"
    CONCEPTDESCRIPTION = "ConceptDescription"
    DATAELEMENT = "DataElement"
    ENTITY = "Entity"
    EVENTELEMENT = "EventElement"
    FILE = "File"
    FRAGMENTREFERENCE = "FragmentReference"
    GLOBALREFERENCE = "GlobalReference"
    IDENTIFIABLE = "Identifiable"
    MULTILANGUAGEPROPERTY = "MultiLanguageProperty"
    OPERATION = "Operation"
    PROPERTY = "Property"
    RANGE = "Range"
    REFERABLE = "Referable"
    REFERENCEELEMENT = "ReferenceElement"
    RELATIONSHIPELEMENT = "RelationshipElement"
    SUBMODEL = "Submodel"
    SUBMODELELEMENT = "SubmodelElement"
    SUBMODELELEMENTCOLLECTION = "SubmodelElementCollection"
    SUBMODELELEMENTLIST = "SubmodelElementList"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.reference_parent_type import ReferenceParentType

if TYPE_CHECKING:
    from ..models.key import Key


T = TypeVar("T", bound="ReferenceParent")


@_attrs_define
class ReferenceParent:
    """
    Attributes:
        type (ReferenceParentType):
        keys (List['Key']):
    """

    type: ReferenceParentType
    keys: List["Key"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        keys = []
        for keys_item_data in self.keys:
            keys_item = keys_item_data.to_dict()
            keys.append(keys_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "keys": keys,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.key import Key

        d = src_dict.copy()
        type = ReferenceParentType(d.pop("type"))

        keys = []
        _keys = d.pop("keys")
        for keys_item_data in _keys:
            keys_item = Key.from_dict(keys_item_data)

            keys.append(keys_item)

        reference_parent = cls(
            type=type,
            keys=keys,
        )

        reference_parent.additional_properties = d
        return reference_parent

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MessageMessageType(str, Enum):
    ERROR = "Error"
    EXCEPTION = "Exception"
    INFO = "Info"
    UNDEFINED = "Undefined"
    WARNING = "Warning"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.message import Message


T = TypeVar("T", bound="Result")


@_attrs_define
class Result:
    """
    Attributes:
        messages (Union[Unset, List['Message']]):
    """

    messages: Union[Unset, List["Message"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        messages: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.messages, Unset):
            messages = []
            for messages_item_data in self.messages:
                messages_item = messages_item_data.to_dict()
                messages.append(messages_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if messages is not UNSET:
            field_dict["messages"] = messages

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message import Message

        d = src_dict.copy()
        messages = []
        _messages = d.pop("messages", UNSET)
        for messages_item_data in _messages or []:
            messages_item = Message.from_dict(messages_item_data)

            messages.append(messages_item)

        result = cls(
            messages=messages,
        )

        result.additional_properties = d
        return result

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.protocol_information_security_attributes import ProtocolInformationSecurityAttributes


T = TypeVar("T", bound="ProtocolInformation")


@_attrs_define
class ProtocolInformation:
    """
    Attributes:
        href (str):
        endpoint_protocol (Union[Unset, str]):
        endpoint_protocol_version (Union[Unset, List[str]]):
        subprotocol (Union[Unset, str]):
        subprotocol_body (Union[Unset, str]):
        subprotocol_body_encoding (Union[Unset, str]):
        security_attributes (Union[Unset, List['ProtocolInformationSecurityAttributes']]):
    """

    href: str
    endpoint_protocol: Union[Unset, str] = UNSET
    endpoint_protocol_version: Union[Unset, List[str]] = UNSET
    subprotocol: Union[Unset, str] = UNSET
    subprotocol_body: Union[Unset, str] = UNSET
    subprotocol_body_encoding: Union[Unset, str] = UNSET
    security_attributes: Union[Unset, List["ProtocolInformationSecurityAttributes"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        href = self.href

        endpoint_protocol = self.endpoint_protocol

        endpoint_protocol_version: Union[Unset, List[str]] = UNSET
        if not isinstance(self.endpoint_protocol_version, Unset):
            endpoint_protocol_version = self.endpoint_protocol_version

        subprotocol = self.subprotocol

        subprotocol_body = self.subprotocol_body

        subprotocol_body_encoding = self.subprotocol_body_encoding

        security_attributes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.security_attributes, Unset):
            security_attributes = []
            for security_attributes_item_data in self.security_attributes:
                security_attributes_item = security_attributes_item_data.to_dict()
                security_attributes.append(security_attributes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "href": href,
            }
        )
        if endpoint_protocol is not UNSET:
            field_dict["endpointProtocol"] = endpoint_protocol
        if endpoint_protocol_version is not UNSET:
            field_dict["endpointProtocolVersion"] = endpoint_protocol_version
        if subprotocol is not UNSET:
            field_dict["subprotocol"] = subprotocol
        if subprotocol_body is not UNSET:
            field_dict["subprotocolBody"] = subprotocol_body
        if subprotocol_body_encoding is not UNSET:
            field_dict["subprotocolBodyEncoding"] = subprotocol_body_encoding
        if security_attributes is not UNSET:
            field_dict["securityAttributes"] = security_attributes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.protocol_information_security_attributes import ProtocolInformationSecurityAttributes

        d = src_dict.copy()
        href = d.pop("href")

        endpoint_protocol = d.pop("endpointProtocol", UNSET)

        endpoint_protocol_version = cast(List[str], d.pop("endpointProtocolVersion", UNSET))

        subprotocol = d.pop("subprotocol", UNSET)

        subprotocol_body = d.pop("subprotocolBody", UNSET)

        subprotocol_body_encoding = d.pop("subprotocolBodyEncoding", UNSET)

        security_attributes = []
        _security_attributes = d.pop("securityAttributes", UNSET)
        for security_attributes_item_data in _security_attributes or []:
            security_attributes_item = ProtocolInformationSecurityAttributes.from_dict(security_attributes_item_data)

            security_attributes.append(security_attributes_item)

        protocol_information = cls(
            href=href,
            endpoint_protocol=endpoint_protocol,
            endpoint_protocol_version=endpoint_protocol_version,
            subprotocol=subprotocol,
            subprotocol_body=subprotocol_body,
            subprotocol_body_encoding=subprotocol_body_encoding,
            security_attributes=security_attributes,
        )

        protocol_information.additional_properties = d
        return protocol_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ExtensionValueType(str, Enum):
    XSANYURI = "xs:anyURI"
    XSBASE64BINARY = "xs:base64Binary"
    XSBOOLEAN = "xs:boolean"
    XSBYTE = "xs:byte"
    XSDATE = "xs:date"
    XSDATETIME = "xs:dateTime"
    XSDECIMAL = "xs:decimal"
    XSDOUBLE = "xs:double"
    XSDURATION = "xs:duration"
    XSFLOAT = "xs:float"
    XSGDAY = "xs:gDay"
    XSGMONTH = "xs:gMonth"
    XSGMONTHDAY = "xs:gMonthDay"
    XSGYEAR = "xs:gYear"
    XSGYEARMONTH = "xs:gYearMonth"
    XSHEXBINARY = "xs:hexBinary"
    XSINT = "xs:int"
    XSINTEGER = "xs:integer"
    XSLONG = "xs:long"
    XSNEGATIVEINTEGER = "xs:negativeInteger"
    XSNONNEGATIVEINTEGER = "xs:nonNegativeInteger"
    XSNONPOSITIVEINTEGER = "xs:nonPositiveInteger"
    XSPOSITIVEINTEGER = "xs:positiveInteger"
    XSSHORT = "xs:short"
    XSSTRING = "xs:string"
    XSTIME = "xs:time"
    XSUNSIGNEDBYTE = "xs:unsignedByte"
    XSUNSIGNEDINT = "xs:unsignedInt"
    XSUNSIGNEDLONG = "xs:unsignedLong"
    XSUNSIGNEDSHORT = "xs:unsignedShort"

    def __str__(self) -> str:
        return str(self.value)



"""A client library for accessing DotAAS Part 2 | HTTP/REST | Asset Administration Shell Registry Service Specification"""

from .client import AuthenticatedClient, Client

__all__ = (
    "AuthenticatedClient",
    "Client",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.asset_administration_shell_descriptor import AssetAdministrationShellDescriptor


T = TypeVar("T", bound="ShellDescriptorSearchResponse")


@_attrs_define
class ShellDescriptorSearchResponse:
    """
    Attributes:
        total (int):
        hits (List['AssetAdministrationShellDescriptor']):
    """

    total: int
    hits: List["AssetAdministrationShellDescriptor"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        total = self.total

        hits = []
        for hits_item_data in self.hits:
            hits_item = hits_item_data.to_dict()
            hits.append(hits_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total": total,
                "hits": hits,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.asset_administration_shell_descriptor import AssetAdministrationShellDescriptor

        d = src_dict.copy()
        total = d.pop("total")

        hits = []
        _hits = d.pop("hits")
        for hits_item_data in _hits:
            hits_item = AssetAdministrationShellDescriptor.from_dict(hits_item_data)

            hits.append(hits_item)

        shell_descriptor_search_response = cls(
            total=total,
            hits=hits,
        )

        shell_descriptor_search_response.additional_properties = d
        return shell_descriptor_search_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .administrative_information import AdministrativeInformation
from .asset_administration_shell_descriptor import AssetAdministrationShellDescriptor
from .asset_administration_shell_descriptor_asset_kind import AssetAdministrationShellDescriptorAssetKind
from .data_specification_content import DataSpecificationContent
from .delete_asset_administration_shell_descriptor_by_id_aas_identifier import (
    DeleteAssetAdministrationShellDescriptorByIdAasIdentifier,
)
from .delete_submodel_descriptor_by_id_through_superpath_aas_identifier import (
    DeleteSubmodelDescriptorByIdThroughSuperpathAasIdentifier,
)
from .delete_submodel_descriptor_by_id_through_superpath_submodel_identifier import (
    DeleteSubmodelDescriptorByIdThroughSuperpathSubmodelIdentifier,
)
from .embedded_data_specification import EmbeddedDataSpecification
from .endpoint import Endpoint
from .extension import Extension
from .extension_value_type import ExtensionValueType
from .get_all_asset_administration_shell_descriptors_asset_kind import (
    GetAllAssetAdministrationShellDescriptorsAssetKind,
)
from .get_all_submodel_descriptors_through_superpath_aas_identifier import (
    GetAllSubmodelDescriptorsThroughSuperpathAasIdentifier,
)
from .get_asset_administration_shell_descriptor_by_id_aas_identifier import (
    GetAssetAdministrationShellDescriptorByIdAasIdentifier,
)
from .get_asset_administration_shell_descriptors_result import GetAssetAdministrationShellDescriptorsResult
from .get_submodel_descriptor_by_id_through_superpath_aas_identifier import (
    GetSubmodelDescriptorByIdThroughSuperpathAasIdentifier,
)
from .get_submodel_descriptor_by_id_through_superpath_submodel_identifier import (
    GetSubmodelDescriptorByIdThroughSuperpathSubmodelIdentifier,
)
from .get_submodel_descriptors_result import GetSubmodelDescriptorsResult
from .key import Key
from .key_type import KeyType
from .lang_string_name_type import LangStringNameType
from .lang_string_text_type import LangStringTextType
from .message import Message
from .message_message_type import MessageMessageType
from .page import Page
from .paged_result_paging_metadata import PagedResultPagingMetadata
from .post_submodel_descriptor_through_superpath_aas_identifier import (
    PostSubmodelDescriptorThroughSuperpathAasIdentifier,
)
from .protocol_information import ProtocolInformation
from .protocol_information_security_attributes import ProtocolInformationSecurityAttributes
from .protocol_information_security_attributes_type import ProtocolInformationSecurityAttributesType
from .put_asset_administration_shell_descriptor_by_id_aas_identifier import (
    PutAssetAdministrationShellDescriptorByIdAasIdentifier,
)
from .put_submodel_descriptor_by_id_through_superpath_aas_identifier import (
    PutSubmodelDescriptorByIdThroughSuperpathAasIdentifier,
)
from .put_submodel_descriptor_by_id_through_superpath_submodel_identifier import (
    PutSubmodelDescriptorByIdThroughSuperpathSubmodelIdentifier,
)
from .reference import Reference
from .reference_parent import ReferenceParent
from .reference_parent_type import ReferenceParentType
from .reference_type import ReferenceType
from .result import Result
from .service_description import ServiceDescription
from .service_description_profiles_item import ServiceDescriptionProfilesItem
from .shell_descriptor_query import ShellDescriptorQuery
from .shell_descriptor_query_query_type import ShellDescriptorQueryQueryType
from .shell_descriptor_search_request import ShellDescriptorSearchRequest
from .shell_descriptor_search_response import ShellDescriptorSearchResponse
from .sorting import Sorting
from .sorting_direction import SortingDirection
from .sorting_path_item import SortingPathItem
from .specific_asset_id import SpecificAssetId
from .submodel_descriptor import SubmodelDescriptor

__all__ = (
    "AdministrativeInformation",
    "AssetAdministrationShellDescriptor",
    "AssetAdministrationShellDescriptorAssetKind",
    "DataSpecificationContent",
    "DeleteAssetAdministrationShellDescriptorByIdAasIdentifier",
    "DeleteSubmodelDescriptorByIdThroughSuperpathAasIdentifier",
    "DeleteSubmodelDescriptorByIdThroughSuperpathSubmodelIdentifier",
    "EmbeddedDataSpecification",
    "Endpoint",
    "Extension",
    "ExtensionValueType",
    "GetAllAssetAdministrationShellDescriptorsAssetKind",
    "GetAllSubmodelDescriptorsThroughSuperpathAasIdentifier",
    "GetAssetAdministrationShellDescriptorByIdAasIdentifier",
    "GetAssetAdministrationShellDescriptorsResult",
    "GetSubmodelDescriptorByIdThroughSuperpathAasIdentifier",
    "GetSubmodelDescriptorByIdThroughSuperpathSubmodelIdentifier",
    "GetSubmodelDescriptorsResult",
    "Key",
    "KeyType",
    "LangStringNameType",
    "LangStringTextType",
    "Message",
    "MessageMessageType",
    "Page",
    "PagedResultPagingMetadata",
    "PostSubmodelDescriptorThroughSuperpathAasIdentifier",
    "ProtocolInformation",
    "ProtocolInformationSecurityAttributes",
    "ProtocolInformationSecurityAttributesType",
    "PutAssetAdministrationShellDescriptorByIdAasIdentifier",
    "PutSubmodelDescriptorByIdThroughSuperpathAasIdentifier",
    "PutSubmodelDescriptorByIdThroughSuperpathSubmodelIdentifier",
    "Reference",
    "ReferenceParent",
    "ReferenceParentType",
    "ReferenceType",
    "Result",
    "ServiceDescription",
    "ServiceDescriptionProfilesItem",
    "ShellDescriptorQuery",
    "ShellDescriptorQueryQueryType",
    "ShellDescriptorSearchRequest",
    "ShellDescriptorSearchResponse",
    "Sorting",
    "SortingDirection",
    "SortingPathItem",
    "SpecificAssetId",
    "SubmodelDescriptor",
)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.shell_descriptor_query_query_type import ShellDescriptorQueryQueryType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ShellDescriptorQuery")


@_attrs_define
class ShellDescriptorQuery:
    """
    Attributes:
        path (str):
        value (str):
        extension_name (Union[Unset, str]): If this property is set, the query applies only to the extension of this
            name. In this case, the path must reference the value property of the extension object.
        query_type (Union[Unset, ShellDescriptorQueryQueryType]):
        combined_with (Union[Unset, ShellDescriptorQuery]):
    """

    path: str
    value: str
    extension_name: Union[Unset, str] = UNSET
    query_type: Union[Unset, ShellDescriptorQueryQueryType] = UNSET
    combined_with: Union[Unset, "ShellDescriptorQuery"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path

        value = self.value

        extension_name = self.extension_name

        query_type: Union[Unset, str] = UNSET
        if not isinstance(self.query_type, Unset):
            query_type = self.query_type.value

        combined_with: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.combined_with, Unset):
            combined_with = self.combined_with.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "path": path,
                "value": value,
            }
        )
        if extension_name is not UNSET:
            field_dict["extensionName"] = extension_name
        if query_type is not UNSET:
            field_dict["queryType"] = query_type
        if combined_with is not UNSET:
            field_dict["combinedWith"] = combined_with

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("path")

        value = d.pop("value")

        extension_name = d.pop("extensionName", UNSET)

        _query_type = d.pop("queryType", UNSET)
        query_type: Union[Unset, ShellDescriptorQueryQueryType]
        if isinstance(_query_type, Unset):
            query_type = UNSET
        else:
            query_type = ShellDescriptorQueryQueryType(_query_type)

        _combined_with = d.pop("combinedWith", UNSET)
        combined_with: Union[Unset, ShellDescriptorQuery]
        if isinstance(_combined_with, Unset):
            combined_with = UNSET
        else:
            combined_with = ShellDescriptorQuery.from_dict(_combined_with)

        shell_descriptor_query = cls(
            path=path,
            value=value,
            extension_name=extension_name,
            query_type=query_type,
            combined_with=combined_with,
        )

        shell_descriptor_query.additional_properties = d
        return shell_descriptor_query

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.sorting_direction import SortingDirection
from ..models.sorting_path_item import SortingPathItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="Sorting")


@_attrs_define
class Sorting:
    """
    Attributes:
        path (List[SortingPathItem]):
        direction (Union[Unset, SortingDirection]):
    """

    path: List[SortingPathItem]
    direction: Union[Unset, SortingDirection] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = []
        for path_item_data in self.path:
            path_item = path_item_data.value
            path.append(path_item)

        direction: Union[Unset, str] = UNSET
        if not isinstance(self.direction, Unset):
            direction = self.direction.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "path": path,
            }
        )
        if direction is not UNSET:
            field_dict["direction"] = direction

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = []
        _path = d.pop("path")
        for path_item_data in _path:
            path_item = SortingPathItem(path_item_data)

            path.append(path_item)

        _direction = d.pop("direction", UNSET)
        direction: Union[Unset, SortingDirection]
        if isinstance(_direction, Unset):
            direction = UNSET
        else:
            direction = SortingDirection(_direction)

        sorting = cls(
            path=path,
            direction=direction,
        )

        sorting.additional_properties = d
        return sorting

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.get_all_asset_administration_shell_descriptors_asset_kind import (
    GetAllAssetAdministrationShellDescriptorsAssetKind,
)
from ...models.get_asset_administration_shell_descriptors_result import GetAssetAdministrationShellDescriptorsResult
from ...models.result import Result
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    limit: Union[Unset, int] = UNSET,
    cursor: Union[Unset, str] = UNSET,
    asset_kind: Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind] = UNSET,
    asset_type: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["limit"] = limit

    params["cursor"] = cursor

    json_asset_kind: Union[Unset, str] = UNSET
    if not isinstance(asset_kind, Unset):
        json_asset_kind = asset_kind.value

    params["assetKind"] = json_asset_kind

    params["assetType"] = asset_type

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/shell-descriptors",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[GetAssetAdministrationShellDescriptorsResult, Result]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = GetAssetAdministrationShellDescriptorsResult.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = Result.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = Result.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = Result.from_dict(response.json())

        return response_500
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[GetAssetAdministrationShellDescriptorsResult, Result]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, int] = UNSET,
    cursor: Union[Unset, str] = UNSET,
    asset_kind: Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind] = UNSET,
    asset_type: Union[Unset, str] = UNSET,
) -> Response[Union[GetAssetAdministrationShellDescriptorsResult, Result]]:
    """Returns all Asset Administration Shell Descriptors

    Args:
        limit (Union[Unset, int]):
        cursor (Union[Unset, str]):
        asset_kind (Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind]):
        asset_type (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetAssetAdministrationShellDescriptorsResult, Result]]
    """

    kwargs = _get_kwargs(
        limit=limit,
        cursor=cursor,
        asset_kind=asset_kind,
        asset_type=asset_type,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, int] = UNSET,
    cursor: Union[Unset, str] = UNSET,
    asset_kind: Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind] = UNSET,
    asset_type: Union[Unset, str] = UNSET,
) -> Optional[Union[GetAssetAdministrationShellDescriptorsResult, Result]]:
    """Returns all Asset Administration Shell Descriptors

    Args:
        limit (Union[Unset, int]):
        cursor (Union[Unset, str]):
        asset_kind (Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind]):
        asset_type (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetAssetAdministrationShellDescriptorsResult, Result]
    """

    return sync_detailed(
        client=client,
        limit=limit,
        cursor=cursor,
        asset_kind=asset_kind,
        asset_type=asset_type,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, int] = UNSET,
    cursor: Union[Unset, str] = UNSET,
    asset_kind: Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind] = UNSET,
    asset_type: Union[Unset, str] = UNSET,
) -> Response[Union[GetAssetAdministrationShellDescriptorsResult, Result]]:
    """Returns all Asset Administration Shell Descriptors

    Args:
        limit (Union[Unset, int]):
        cursor (Union[Unset, str]):
        asset_kind (Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind]):
        asset_type (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[GetAssetAdministrationShellDescriptorsResult, Result]]
    """

    kwargs = _get_kwargs(
        limit=limit,
        cursor=cursor,
        asset_kind=asset_kind,
        asset_type=asset_type,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    limit: Union[Unset, int] = UNSET,
    cursor: Union[Unset, str] = UNSET,
    asset_kind: Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind] = UNSET,
    asset_type: Union[Unset, str] = UNSET,
) -> Optional[Union[GetAssetAdministrationShellDescriptorsResult, Result]]:
    """Returns all Asset Administration Shell Descriptors

    Args:
        limit (Union[Unset, int]):
        cursor (Union[Unset, str]):
        asset_kind (Union[Unset, GetAllAssetAdministrationShellDescriptorsAssetKind]):
        asset_type (Union[Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[GetAssetAdministrationShellDescriptorsResult, Result]
    """

    return (
        await asyncio_detailed(
            client=client,
            limit=limit,
            cursor=cursor,
            asset_kind=asset_kind,
            asset_type=asset_type,
        )
    ).parsed
